@@ -8,6 +8,7 @@ from .field import Field
 from .line import Line
 from .operators import DValue, dx, dy, dz, dV, dS, integral
 from .interact import interact
+from .field_rotator import FieldRotator
 import matplotlib.pyplot as plt
 
 # Enable default plotting style.
@@ -16,7 +17,6 @@ path = os.path.join(dirname, './util/plotting-style.mplstyle')
 plt.style.use(path)
 
 __version__ = pkg_resources.get_distribution(__name__).version
-__dependencies__ = pkg_resources.require(__name__)
 
 
 def test():
