@@ -1,8 +1,10 @@
 import k3d
+import copy
 import itertools
 import ipywidgets
 import collections
 import numpy as np
+import warnings
 import discretisedfield as df
 import ubermagutil.units as uu
 import matplotlib.pyplot as plt
@@ -15,11 +17,7 @@ from mpl_toolkits.mplot3d import Axes3D
                cell=ts.Vector(size=3, positive=True, const=True),
                n=ts.Vector(size=3, component_type=int, unsigned=True,
                            const=True),
-               bc=ts.Typed(expected_type=str),
-               subregions=ts.Dictionary(
-                   key_descriptor=ts.Name(),
-                   value_descriptor=ts.Typed(expected_type=df.Region),
-                   allow_empty=True))
+               bc=ts.Typed(expected_type=str))
 class Mesh:
     """Finite-difference mesh.
 
@@ -169,7 +167,7 @@ class Mesh:
 
     """
     def __init__(self, *, region=None, p1=None, p2=None, n=None, cell=None,
-                 bc='', subregions=dict()):
+                 bc='', subregions=dict(), attributes={'unit': 'm'}):
         if region is not None and p1 is None and p2 is None:
             self.region = region
         elif region is None and p1 is not None and p2 is not None:
@@ -191,7 +189,7 @@ class Mesh:
             raise ValueError(msg)
 
         # Check if the mesh region is an aggregate of the discretisation cell.
-        tol = 1e-12  # picometre tolerance
+        tol = np.min(self.cell) * 1e-3  # tolerance
         rem = np.remainder(self.region.edges, self.cell)
         if np.logical_and(np.greater(rem, tol),
                           np.less(rem, np.subtract(self.cell, tol))).any():
@@ -201,6 +199,31 @@ class Mesh:
 
         self.bc = bc.lower()
 
+        self.subregions = subregions
+
+        self.attributes = attributes
+        if 'fourierspace' not in attributes.keys():
+            self.attributes['fourierspace'] = False
+        if 'isplane' not in attributes.keys():
+            self.attributes['isplane'] = False
+
+    @property
+    def subregions(self):
+        """"Subregions of the mesh.
+
+        Returns
+        -------
+        dict
+
+            A dictionary defining subregions in the mesh. The keys of the
+            dictionary are the region names (``str``) as valid Python variable
+            names, whereas the values are ``discretisedfield.Region`` objects.
+            Defaults to an empty dictionary.
+        """
+        return self._subregions
+
+    @subregions.setter
+    def subregions(self, subregions):
         # Check if subregions are aligned with the mesh
         for key, value in subregions.items():
             # Is the subregion in the mesh region?
@@ -210,7 +233,7 @@ class Mesh:
 
             # Is the subregion an aggregate of discretisation cell?
             try:
-                submesh = self.__class__(region=value, cell=self.cell)
+                self.__class__(region=value, cell=self.cell)
             except ValueError:
                 msg = (f'Subregion {key} cannot be divided into '
                        f'discretisation cells of size {self.cell=}.')
@@ -221,8 +244,19 @@ class Mesh:
                     self.__class__(region=value, cell=self.cell)):
                 msg = f'Subregion {key} is not aligned with the mesh.'
                 raise ValueError(msg)
+        if 'default' in subregions.keys():
+            msg = ('Subregion name ``default`` has a special meaning when '
+                   'initialising field values')
+            warnings.warn(msg)
+        self._subregions = subregions
 
-        self.subregions = subregions
+    @property
+    def attributes(self):
+        return self._attributes
+
+    @attributes.setter
+    def attributes(self, attributes):
+        self._attributes = copy.deepcopy(attributes)
 
     def __len__(self):
         """Number of discretisation cells in the mesh.
@@ -431,7 +465,8 @@ class Mesh:
 
         """
         return (f'Mesh(region={repr(self.region)}, n={self.n}, '
-                f'bc=\'{self.bc}\', subregions={self.subregions})')
+                f'bc=\'{self.bc}\', subregions={self.subregions},'
+                f' attributes={self.attributes})')
 
     def index2point(self, index, /):
         """Convert cell's index to its coordinate.
@@ -744,14 +779,15 @@ class Mesh:
                        point + self.cell[planeaxis]/2)
         npm[ilist] = (*n, 1)
 
-        plane_mesh = self.__class__(p1=p1pm, p2=p2pm, n=dfu.array2tuple(npm))
+        attributes = self.attributes
 
-        # Add info dictionary, so that the mesh can be interpreted easier.
-        info = dict()
-        info['planeaxis'] = planeaxis
-        info['point'] = point
-        info['axis1'], info['axis2'] = axis1, axis2
-        plane_mesh.info = info
+        plane_mesh = self.__class__(p1=p1pm, p2=p2pm, n=dfu.array2tuple(npm),
+                                    attributes=attributes)
+        plane_mesh.attributes['isplane'] = True
+        plane_mesh.attributes['planeaxis'] = planeaxis
+        plane_mesh.attributes['point'] = point
+        plane_mesh.attributes['axis1'] = axis1
+        plane_mesh.attributes['axis2'] = axis2
 
         return plane_mesh
 
@@ -889,7 +925,8 @@ class Mesh:
 
         """
         if isinstance(item, str):
-            return self.__class__(region=self.subregions[item], cell=self.cell)
+            return self.__class__(region=self.subregions[item], cell=self.cell,
+                                  attributes=self.attributes)
 
         if item not in self.region:
             msg = 'Subregion is outside the mesh region.'
@@ -899,7 +936,8 @@ class Mesh:
         p1 = np.subtract(self.index2point(self.point2index(item.pmin)), hc)
         p2 = np.add(self.index2point(self.point2index(item.pmax)), hc)
 
-        return self.__class__(region=df.Region(p1=p1, p2=p2), cell=self.cell)
+        return self.__class__(region=df.Region(p1=p1, p2=p2), cell=self.cell,
+                              attributes=self.attributes)
 
     def pad(self, pad_width):
         """Mesh padding.
@@ -957,7 +995,8 @@ class Mesh:
             pmin[axis] -= pad_width[direction][0] * self.cell[axis]
             pmax[axis] += pad_width[direction][1] * self.cell[axis]
 
-        return self.__class__(p1=pmin, p2=pmax, cell=self.cell, bc=self.bc)
+        return self.__class__(p1=pmin, p2=pmax, cell=self.cell, bc=self.bc,
+                              attributes=self.attributes)
 
     def __getattr__(self, attr):
         """Extracting the discretisation in a particular direction.
@@ -1074,12 +1113,13 @@ class Mesh:
         (0.0, 0.0, 2.0)
 
         """
-        if not hasattr(self, 'info'):
+        if not self.attributes['isplane']:
             msg = 'The mesh must be sliced before dS can be computed.'
             raise ValueError(msg)
 
-        norm = self.cell[self.info['axis1']] * self.cell[self.info['axis2']]
-        dn = dfu.assemble_index(0, 3, {self.info['planeaxis']: 1})
+        norm = (self.cell[self.attributes['axis1']]
+                * self.cell[self.attributes['axis2']])
+        dn = dfu.assemble_index(0, 3, {self.attributes['planeaxis']: 1})
         return df.Field(self, dim=3, value=dn, norm=norm)
 
     def mpl(self, *, ax=None, figsize=None, color=dfu.cp_hex[:2],
@@ -1165,14 +1205,16 @@ class Mesh:
             plt.savefig(filename, bbox_inches='tight', pad_inches=0)
 
     def mpl_subregions(self, *, ax=None, figsize=None, color=dfu.cp_hex,
-                       multiplier=None, filename=None, **kwargs):
+                       multiplier=None, filename=None, show_region=False,
+                       **kwargs):
         """``matplotlib`` subregions plot.
 
         If ``ax`` is not passed, ``matplotlib.axes.Axes`` object is created
         automatically and the size of a figure can be specified using
         ``figsize``. The color of lines depicting subregions and can be
         specified using ``color`` list. The plot is saved in PDF-format if
-        ``filename`` is passed.
+        ``filename`` is passed. The whole region is only shown if
+        ``show_region=True``.
 
         It is often the case that the object size is either small (e.g. on a
         nanoscale) or very large (e.g. in units of kilometers). Accordingly,
@@ -1212,6 +1254,10 @@ class Mesh:
 
             If filename is passed, the plot is saved. Defaults to ``None``.
 
+        show_region : bool, optional
+
+            If ``True`` also plot the whole region. Defaults to ``False``.
+
         Examples
         --------
         1. Visualising subregions using ``matplotlib``.
@@ -1235,6 +1281,9 @@ class Mesh:
         if multiplier is None:
             multiplier = uu.si_max_multiplier(self.region.edges)
 
+        if show_region:
+            self.region.mpl(ax=ax, multiplier=multiplier, color='grey')
+
         for i, subregion in enumerate(self.subregions.values()):
             subregion.mpl(ax=ax, multiplier=multiplier,
                           color=color[i % len(color)], **kwargs)
