@@ -47,10 +47,11 @@ def check_mesh(mesh):
     assert all(isinstance(i, tuple) for i in line)
     assert all(i in mesh.region for i in line)
 
+    assert isinstance(mesh.attributes, dict)
+
     plane_mesh = mesh.plane('z', n=(2, 2))
     assert isinstance(plane_mesh, df.Mesh)
-    assert isinstance(plane_mesh.info, dict)
-    assert plane_mesh.info
+    assert plane_mesh.attributes['isplane']
     assert 1 in plane_mesh.n
     assert len(plane_mesh) == 4
     assert all(isinstance(i, tuple) for i in plane_mesh)
@@ -272,9 +273,10 @@ class TestMesh:
         mesh = df.Mesh(p1=p1, p2=p2, cell=cell, bc='x')
         check_mesh(mesh)
 
-        rstr = ('Mesh(region=Region(p1=(-1.0, -4.0, 11.0), '
-                'p2=(15.0, 10.1, 12.5)), n=(16, 141, 3), '
-                'bc=\'x\', subregions={})')
+        rstr = ("Mesh(region=Region(p1=(-1.0, -4.0, 11.0), "
+                "p2=(15.0, 10.1, 12.5)), n=(16, 141, 3), "
+                "bc='x', subregions={}, attributes={'unit': 'm', "
+                "'fourierspace': False, 'isplane': False})")
         assert repr(mesh) == rstr
 
     def test_index2point(self):
@@ -512,43 +514,50 @@ class TestMesh:
         with pytest.raises(ValueError):
             plane = list(mesh.plane(x=2, z=1))
 
-        info = mesh.plane('x').info
+        info = mesh.plane('x').attributes
+        assert info['isplane']
         assert info['planeaxis'] == 0
         assert info['axis1'] == 1
         assert info['axis2'] == 2
         assert info['point'] == 5
 
-        info = mesh.plane('y').info
+        info = mesh.plane('y').attributes
+        assert info['isplane']
         assert info['planeaxis'] == 1
         assert info['axis1'] == 0
         assert info['axis2'] == 2
         assert info['point'] == 2.5
 
-        info = mesh.plane('z').info
+        info = mesh.plane('z').attributes
+        assert info['isplane']
         assert info['planeaxis'] == 2
         assert info['axis1'] == 0
         assert info['axis2'] == 1
         assert info['point'] == 1.5
 
-        info = mesh.plane(x=0).info
+        info = mesh.plane(x=0).attributes
+        assert info['isplane']
         assert info['planeaxis'] == 0
         assert info['axis1'] == 1
         assert info['axis2'] == 2
         assert info['point'] == 0
 
-        info = mesh.plane(y=0).info
+        info = mesh.plane(y=0).attributes
+        assert info['isplane']
         assert info['planeaxis'] == 1
         assert info['axis1'] == 0
         assert info['axis2'] == 2
         assert info['point'] == 0
 
-        info = mesh.plane(z=0).info
+        info = mesh.plane(z=0).attributes
+        assert info['isplane']
         assert info['planeaxis'] == 2
         assert info['axis1'] == 0
         assert info['axis2'] == 1
         assert info['point'] == 0
 
-        info = mesh.plane(x=5).info
+        info = mesh.plane(x=5).attributes
+        assert info['isplane']
         assert info['planeaxis'] == 0
         assert info['axis1'] == 1
         assert info['axis2'] == 2
