@@ -232,6 +232,44 @@ class TestRegion:
         with pytest.raises(TypeError):
             res = region1 | 5
 
+    def test_multiplier(self):
+        p1 = (-50e-9, -50e-9, 0)
+        p2 = (50e-9, 50e-9, 20e-9)
+        region = df.Region(p1=p1, p2=p2)
+
+        assert region.multiplier == 1e-9
+
+        p1 = (0, 0, 0)
+        p2 = (1e-5, 1e-4, 1e-5)
+        region = df.Region(p1=p1, p2=p2)
+
+        assert region.multiplier == 1e-6
+
+    def test_mul_truediv(self):
+        p1 = (-50e-9, -50e-9, 0)
+        p2 = (50e-9, 50e-9, 20e-9)
+        region = df.Region(p1=p1, p2=p2)
+
+        res = region * 2
+        check_region(res)
+        assert np.allclose(res.pmin, (-100e-9, -100e-9, 0))
+        assert np.allclose(res.pmax, (100e-9, 100e-9, 40e-9))
+        assert np.allclose(res.edges, (200e-9, 200e-9, 40e-9))
+
+        res = region / 2
+        check_region(res)
+        assert np.allclose(res.pmin, (-25e-9, -25e-9, 0))
+        assert np.allclose(res.pmax, (25e-9, 25e-9, 10e-9))
+        assert np.allclose(res.edges, (50e-9, 50e-9, 10e-9))
+
+        assert region * 2 == 2 * region == region / 0.5
+
+        with pytest.raises(TypeError):
+            res = region * region
+
+        with pytest.raises(TypeError):
+            res = 5 / region
+
     def test_mpl(self):
         p1 = (-50e-9, -50e-9, 0)
         p2 = (50e-9, 50e-9, 20e-9)
@@ -239,7 +277,7 @@ class TestRegion:
 
         check_region(region)
 
-        # Check if runs.
+        # Check if it runs.
         region.mpl()
         region.mpl(figsize=(10, 10), multiplier=1e-9, color=dfu.cp_hex[1],
                    linewidth=3, linestyle='dashed')
